@@ -41,7 +41,6 @@ const PopupMenu = imports.ui.popupMenu;
 const Tweener = imports.ui.tweener;
 
 const Config = Extension.imports.config;
-const Settings = Extension.imports.settings;
 const Timer = Extension.imports.timer;
 
 const Gettext = imports.gettext.domain(Config.GETTEXT_PACKAGE);
@@ -56,6 +55,12 @@ const FADE_OUT_OPACITY = 0.38;
 
 const STEPS = 120;
 
+const IndicatorType = {
+    TEXT: 'text',
+    SHORT_TEXT: 'short-text',
+    ICON: 'icon'
+};
+
 
 const IndicatorMenu = new Lang.Class({
     Name: 'PomodoroIndicatorMenu',
@@ -75,17 +80,18 @@ const IndicatorMenu = new Lang.Class({
         this._createTimerMenuItem();
 
         this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
-        this.addAction(_("Preferences"), Lang.bind(this, this._showPreferences));
+        this.addAction(_("Preferences"), Lang.bind(this, this._activatePreferences));
+        this.addAction(_("Quit"), Lang.bind(this, this._activateQuit));
     },
 
     _createActionButton: function(iconName, accessibleName) {
-        let icon = new St.Button({ reactive: true,
-                                   can_focus: true,
-                                   track_hover: true,
-                                   accessible_name: accessibleName,
-                                   style_class: 'system-menu-action extension-pomodoro-menu-action' });
-        icon.child = new St.Icon({ icon_name: iconName });
-        return icon;
+        let button = new St.Button({ reactive: true,
+                                     can_focus: true,
+                                     track_hover: true,
+                                     accessible_name: accessibleName,
+                                     style_class: 'system-menu-action extension-pomodoro-menu-action' });
+        button.child = new St.Icon({ icon_name: iconName });
+        return button;
     },
 
     _onTimerClicked: function() {
@@ -143,6 +149,7 @@ const IndicatorMenu = new Lang.Class({
         hbox = new St.BoxLayout();
 
         this._startAction = this._createActionButton('media-playback-start-symbolic', _("Start Timer"));
+        this._startAction.add_style_class_name('extension-pomodoro-menu-action-border');
         this._startAction.connect('clicked', Lang.bind(this, this._onStartClicked));
         hbox.add_actor(this._startAction);
 
@@ -261,12 +268,16 @@ const IndicatorMenu = new Lang.Class({
         return '%02d:%02d'.format(minutes, seconds);
     },
 
-    _showPreferences: function() {
+    _activatePreferences: function() {
         let timestamp = global.get_current_time();
 
         this.indicator.timer.showPreferences(timestamp);
     },
 
+    _activateQuit: function() {
+        this.indicator.timer.quit();
+    },
+
     _activateState: function(stateName) {
         this.indicator.timer.setState(stateName);
     },
@@ -289,8 +300,224 @@ const IndicatorMenu = new Lang.Class({
 });
 
 
-const IndicatorIcon = new Lang.Class({
-    Name: 'PomodoroIndicatorIcon',
+const TextIndicator = new Lang.Class({
+    Name: 'PomodoroTextIndicator',
+
+    _init : function(timer) {
+        this._initialized     = false;
+        this._state           = Timer.State.NULL;
+        this._minHPadding     = 0;
+        this._natHPadding     = 0;
+        this._digitWidth      = 0;
+        this._charWidth       = 0;
+        this._onTimerUpdateId = 0;
+
+        this.timer = timer;
+
+        this.actor = new Shell.GenericContainer({ reactive: true });
+        this.actor._delegate = this;
+
+        this.label = new St.Label({ style_class: 'system-status-label',
+                                    x_align: Clutter.ActorAlign.CENTER,
+                                    y_align: Clutter.ActorAlign.CENTER });
+        this.label.clutter_text.line_wrap = false;
+        this.label.clutter_text.ellipsize = false;
+        this.label.connect('destroy', Lang.bind(this,
+            function() {
+                if (this._onTimerUpdateId) {
+                    this.timer.disconnect(this._onTimerUpdateId);
+                    this._onTimerUpdateId = 0;
+                }
+            }));
+        this.actor.add_child(this.label);
+
+        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        this.actor.connect('allocate', Lang.bind(this, this._allocate));
+        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
+        this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
+
+        this._onTimerUpdateId = this.timer.connect('update', Lang.bind(this, this._onTimerUpdate));
+
+        this._onTimerUpdate();
+
+        this._state = this.timer.getState();
+        this._initialized = true;
+
+        if (this._state == Timer.State.POMODORO) {
+            this.actor.set_opacity(FADE_IN_OPACITY * 255);
+        }
+        else {
+            this.actor.set_opacity(FADE_OUT_OPACITY * 255);
+        }
+    },
+
+    _onStyleChanged: function(actor) {
+        let themeNode = actor.get_theme_node();
+        let font      = themeNode.get_font();
+        let context   = actor.get_pango_context();
+        let metrics   = context.get_metrics(font, context.get_language());
+
+        this._minHPadding = themeNode.get_length('-minimum-hpadding');
+        this._natHPadding = themeNode.get_length('-natural-hpadding');
+        this._digitWidth  = metrics.get_approximate_digit_width() / Pango.SCALE;
+        this._charWidth   = metrics.get_approximate_char_width() / Pango.SCALE;
+    },
+
+    _getWidth: function() {
+        return Math.ceil(4 * this._digitWidth + 0.5 * this._charWidth);
+    },
+
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        let child        = actor.get_first_child();
+        let minWidth     = this._getWidth();
+        let naturalWidth = minWidth;
+
+        minWidth     += 2 * this._minHPadding;
+        naturalWidth += 2 * this._natHPadding;
+
+        if (child) {
+            [alloc.min_size, alloc.natural_size] = child.get_preferred_width(-1);
+        }
+        else {
+            alloc.min_size = alloc.natural_size = 0;
+        }
+
+        if (alloc.min_size < minWidth) {
+            alloc.min_size = minWidth;
+        }
+
+        if (alloc.natural_size < naturalWidth) {
+            alloc.natural_size = naturalWidth;
+        }
+    },
+
+    _getPreferredHeight: function(actor, forWidth, alloc) {
+        let child = actor.get_first_child();
+
+        if (child) {
+            [alloc.min_size, alloc.natural_size] = child.get_preferred_height(-1);
+        }
+        else {
+            alloc.min_size = alloc.natural_size = 0;
+        }
+    },
+
+    _getText: function(state, remaining) {
+        if (remaining < 0.0) {
+            remaining = 0.0;
+        }
+
+        let minutes = Math.floor(remaining / 60);
+        let seconds = Math.floor(remaining % 60);
+
+        return '%02d:%02d'.format(minutes, seconds);
+    },
+
+    _onTimerUpdate: function() {
+        let state = this.timer.getState();
+        let remaining = this.timer.getRemaining();
+
+        if (this._state != state && this._initialized) {
+            this._state = state;
+
+            if (state == Timer.State.POMODORO) {
+                Tweener.addTween(this.actor,
+                                 { opacity: FADE_IN_OPACITY * 255,
+                                   time: FADE_IN_TIME / 1000,
+                                   transition: 'easeOutQuad' });
+            }
+            else {
+                Tweener.addTween(this.actor,
+                                 { opacity: FADE_OUT_OPACITY * 255,
+                                   time: FADE_OUT_TIME / 1000,
+                                   transition: 'easeOutQuad' });
+            }
+        }
+
+        this.label.set_text(this._getText(state, remaining));
+    },
+
+    _allocate: function(actor, box, flags) {
+        let child = actor.get_first_child();
+        if (!child)
+            return;
+
+        let [minWidth, natWidth] = child.get_preferred_width(-1);
+
+        let availWidth  = box.x2 - box.x1;
+        let availHeight = box.y2 - box.y1;
+
+        let childBox = new Clutter.ActorBox();
+        childBox.y1 = 0;
+        childBox.y2 = availHeight;
+
+        if (natWidth + 2 * this._natHPadding <= availWidth) {
+            childBox.x1 = this._natHPadding;
+            childBox.x2 = availWidth - this._natHPadding;
+        }
+        else {
+            childBox.x1 = this._minHPadding;
+            childBox.x2 = availWidth - this._minHPadding;
+        }
+
+        child.allocate(childBox, flags);
+    },
+
+    _onActorDestroy: function() {
+        if (this._onTimerUpdateId) {
+            this.timer.disconnect(this._onTimerUpdateId);
+            this._onTimerUpdateId = 0;
+        }
+
+        this.actor._delegate = null;
+
+        this.emit('destroy');
+    },
+
+    destroy: function() {
+        this.actor.destroy();
+    }
+});
+Signals.addSignalMethods(TextIndicator.prototype);
+
+
+const ShortTextIndicator = new Lang.Class({
+    Name: 'PomodoroShortTextIndicator',
+    Extends: TextIndicator,
+
+    _init: function(timer) {
+        this.parent(timer);
+
+        this.label.set_x_align(Clutter.ActorAlign.END);
+    },
+
+    _getWidth: function() {
+        return Math.ceil(2 * this._digitWidth +
+                         1 * this._charWidth);
+    },
+
+    _getText: function(state, remaining) {
+        if (remaining < 0.0) {
+            remaining = 0.0;
+        }
+
+        let minutes = Math.round(remaining / 60);
+        let seconds = Math.round(remaining % 60);
+
+        if (remaining > 15) {
+            seconds = Math.ceil(seconds / 15) * 15;
+        }
+
+        return (remaining > 45)
+                ? "%d′".format(minutes, remaining)
+                : "%d″".format(seconds, remaining);
+    }
+});
+
+
+const IconIndicator = new Lang.Class({
+    Name: 'PomodoroIconIndicator',
 
     _init : function(timer) {
         this._state           = Timer.State.NULL;
@@ -492,18 +719,18 @@ const IndicatorIcon = new Lang.Class({
         this.actor.destroy();
     }
 });
-Signals.addSignalMethods(IndicatorIcon.prototype);
+Signals.addSignalMethods(IconIndicator.prototype);
 
 
 const Indicator = new Lang.Class({
     Name: 'PomodoroIndicator',
     Extends: PanelMenu.Button,
 
-    _init: function(timer) {
+    _init: function(timer, type) {
         this.parent(St.Align.START, _("Pomodoro"), true);
 
-        this.timer = timer;
-        this.icon = new IndicatorIcon(this.timer);
+        this.timer  = timer;
+        this.widget = null;
 
         this.actor.add_style_class_name('extension-pomodoro-indicator');
         this.actor.connect('destroy', Lang.bind(this, this._onActorDestroy));
@@ -513,11 +740,14 @@ const Indicator = new Lang.Class({
         this._blinkTimeoutSource = 0;
 
         this._hbox = new St.BoxLayout({ style_class: 'panel-status-menu-box' });
-        this._hbox.add_actor(this.icon.actor);
-        this._hbox.add_actor(this._arrow);
+        this._hbox.pack_start = true;
+        this._hbox.add_child(this._arrow, { expand: false,
+                                            x_fill: false,
+                                            x_align: St.Align.END });
         this.actor.add_child(this._hbox);
 
         this.setMenu(new IndicatorMenu(this));
+        this.setType(type);
 
         this._onBlinked();
 
@@ -525,6 +755,36 @@ const Indicator = new Lang.Class({
         this._timerResumedId = this.timer.connect('resumed', Lang.bind(this, this._onTimerResumed));
     },
 
+    setType: function(type) {
+        if (this.widget) {
+            this.widget.destroy();
+            this.widget = null;
+        }
+
+        switch (type) {
+            case IndicatorType.TEXT:
+                this.widget = new TextIndicator(this.timer);
+                break;
+
+            case IndicatorType.SHORT_TEXT:
+                this.widget = new ShortTextIndicator(this.timer);
+                break;
+
+            default:
+                this.widget = new IconIndicator(this.timer);
+                break;
+        }
+
+        this.widget.actor.bind_property('opacity',
+                                        this._arrow,
+                                        'opacity',
+                                        GObject.BindingFlags.SYNC_CREATE);
+
+        this._hbox.add_child(this.widget.actor, { expand: false,
+                                                  x_fill: false,
+                                                  x_align: St.Align.START });
+    },
+
     _onBlinked: function() {
         this._blinking = false;
 
